"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Injector = void 0;
var web3_1 = __importDefault(require("web3"));
var path_1 = __importDefault(require("path"));
var core_1 = require("@ethereum-sourcify/core");
var utils_1 = require("../utils");
var multihashes = require('multihashes');
var Injector = /** @class */ (function () {
    /**
     * Constructor
     * @param {InjectorConfig = {}} config
     */
    function Injector(config) {
        if (config === void 0) { config = {}; }
        this.chains = {};
        this.infuraPID = config.infuraPID;
        this.localChainUrl = config.localChainUrl;
        this.offline = config.offline || false;
        this.repositoryPath = config.repositoryPath;
        this.log = config.log || core_1.Logger("Injector");
        this.fileService = config.fileService || new core_1.FileService(this.repositoryPath, this.log);
        if (!this.offline) {
            this.initChains();
        }
    }
    /**
     * Instantiates a web3 provider for all public ethereum networks via Infura or regular node.
     * If environment variable TESTING is set to true, localhost:8545 is also available.
     */
    Injector.prototype.initChains = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _loop_1, this_1, _i, _a, chain, chainOption;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!this.infuraPID) return [3 /*break*/, 2];
                        return [4 /*yield*/, utils_1.checkEndpoint(this.infuraPID).catch(function (err) {
                                _this.log.warn({ infuraID: _this.infuraPID }, err.message);
                            })];
                    case 1:
                        _b.sent();
                        _b.label = 2;
                    case 2:
                        _loop_1 = function (chain) {
                            var web3, web3_2;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        this_1.chains[chain.chainId] = {};
                                        if (!this_1.infuraPID) return [3 /*break*/, 1];
                                        web3 = chain.web3[0].replace('${INFURA_ID}', this_1.infuraPID);
                                        this_1.chains[chain.chainId].web3 = new web3_1.default(web3);
                                        return [3 /*break*/, 3];
                                    case 1:
                                        web3_2 = chain.fullnode.dappnode;
                                        this_1.chains[chain.chainId].web3 = new web3_1.default(web3_2);
                                        return [4 /*yield*/, utils_1.checkEndpoint(web3_2).catch(function () {
                                                _this.log.warn({ endpoint: web3_2 }, "Invalid endpoint for chain " + chain.name);
                                            })];
                                    case 2:
                                        _a.sent();
                                        _a.label = 3;
                                    case 3: return [2 /*return*/];
                                }
                            });
                        };
                        this_1 = this;
                        _i = 0, _a = core_1.getSupportedChains();
                        _b.label = 3;
                    case 3:
                        if (!(_i < _a.length)) return [3 /*break*/, 6];
                        chain = _a[_i];
                        return [5 /*yield**/, _loop_1(chain)];
                    case 4:
                        _b.sent();
                        _b.label = 5;
                    case 5:
                        _i++;
                        return [3 /*break*/, 3];
                    case 6:
                        // For unit testing with testrpc...
                        if (this.localChainUrl) {
                            chainOption = core_1.getChainByName('Localchain');
                            this.chains[chainOption.chainId] = {
                                web3: new web3_1.default(chainOption.web3[0])
                            };
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Searches a set of addresses for the one whose deployedBytecode
     * matches a given bytecode string
     * @param {String[]}          addresses
     * @param {string}      deployedBytecode
     */
    Injector.prototype.matchBytecodeToAddress = function (chain, addresses, compiledBytecode) {
        if (addresses === void 0) { addresses = []; }
        return __awaiter(this, void 0, void 0, function () {
            var match, _i, addresses_1, address, deployedBytecode, e_1, status_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        match = { address: null, status: null };
                        _i = 0, addresses_1 = addresses;
                        _a.label = 1;
                    case 1:
                        if (!(_i < addresses_1.length)) return [3 /*break*/, 7];
                        address = addresses_1[_i];
                        address = web3_1.default.utils.toChecksumAddress(address);
                        deployedBytecode = null;
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        this.log.info({
                            loc: '[MATCH]',
                            chain: chain,
                            address: address
                        }, "Retrieving contract bytecode address");
                        return [4 /*yield*/, utils_1.getBytecode(this.chains[chain].web3, address)];
                    case 3:
                        deployedBytecode = _a.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        e_1 = _a.sent();
                        return [3 /*break*/, 5];
                    case 5:
                        status_1 = this.compareBytecodes(deployedBytecode, compiledBytecode);
                        if (status_1) {
                            match = { address: address, status: status_1 };
                            return [3 /*break*/, 7];
                        }
                        _a.label = 6;
                    case 6:
                        _i++;
                        return [3 /*break*/, 1];
                    case 7: return [2 /*return*/, match];
                }
            });
        });
    };
    /**
     * Returns a string description of how closely two bytecodes match. Bytecodes
     * that match in all respects apart from their metadata hashes are 'partial'.
     * Bytecodes that don't match are `null`.
     * @param  {string} deployedBytecode
     * @param  {string} compiledBytecode
     * @return {string | null}  match description ('perfect'|'partial'|null)
     */
    Injector.prototype.compareBytecodes = function (deployedBytecode, compiledBytecode) {
        if (deployedBytecode && deployedBytecode.length > 2) {
            if (deployedBytecode === compiledBytecode) {
                return 'perfect';
            }
            if (utils_1.getBytecodeWithoutMetadata(deployedBytecode) === utils_1.getBytecodeWithoutMetadata(compiledBytecode)) {
                return 'partial';
            }
        }
        return null;
    };
    /**
     * Throws if addresses array contains a null value (express) or is length 0
     * @param {string[] = []} addresses param (submitted to injector)
     */
    Injector.prototype.validateAddresses = function (addresses) {
        if (addresses === void 0) { addresses = []; }
        var err = new Error("Missing address for submitted sources/metadata");
        if (!addresses.length) {
            throw err;
        }
        for (var _i = 0, addresses_2 = addresses; _i < addresses_2.length; _i++) {
            var address = addresses_2[_i];
            if (address == null)
                throw err;
        }
    };
    /**
     * Throws if `chain` is falsy or wrong type
     * @param {string} chain param (submitted to injector)
     */
    Injector.prototype.validateChain = function (chain) {
        if (!chain || typeof chain !== 'string') {
            throw new Error("Missing chain name for submitted sources/metadata");
            ;
        }
    };
    /**
     * Used by the front-end. Accepts a set of source files and a metadata string,
     * recompiles / validates them and stores them in the repository by chain/address
     * and by swarm | ipfs hash.
     * @param  {string}            repository repository root (ex: 'repository')
     * @param  {string}            chain      chain name (ex: 'ropsten')
     * @param  {string}            address    contract address
     * @param  {string[]}          files
     * @return {Promise<object>}              address & status of successfully verified contract
     */
    Injector.prototype.inject = function (inputData) {
        return __awaiter(this, void 0, void 0, function () {
            var chain, addresses, files, match, _i, files_1, source, target, compilationResult, err_1, status_2, err;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        chain = inputData.chain, addresses = inputData.addresses, files = inputData.files;
                        this.validateAddresses(addresses);
                        this.validateChain(chain);
                        match = {
                            address: null,
                            status: null
                        };
                        _i = 0, files_1 = files;
                        _a.label = 1;
                    case 1:
                        if (!(_i < files_1.length)) return [3 /*break*/, 10];
                        source = files_1[_i];
                        target = Object.assign({}, source.metadata.settings.compilationTarget);
                        compilationResult = void 0;
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, utils_1.recompile(source.metadata, source.solidity, this.log)];
                    case 3:
                        compilationResult = _a.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        err_1 = _a.sent();
                        this.log.info({ loc: "[RECOMPILE]", err: err_1 });
                        throw err_1;
                    case 5:
                        if (!inputData.bytecode) return [3 /*break*/, 6];
                        status_2 = this.compareBytecodes(inputData.bytecode, compilationResult.deployedBytecode);
                        match = {
                            address: web3_1.default.utils.toChecksumAddress(addresses[0]),
                            status: status_2
                        };
                        return [3 /*break*/, 8];
                    case 6: return [4 /*yield*/, this.matchBytecodeToAddress(chain, addresses, compilationResult.deployedBytecode)];
                    case 7:
                        match = _a.sent();
                        _a.label = 8;
                    case 8:
                        // Since the bytecode matches, we can be sure that we got the right
                        // metadata file (up to json formatting) and exactly the right sources.
                        // Now we can store the re-compiled and correctly formatted metadata file
                        // and the sources.
                        if (match.address && match.status === 'perfect') {
                            this.storePerfectMatchData(this.repositoryPath, chain, match.address, compilationResult, source.solidity);
                        }
                        else if (match.address && match.status === 'partial') {
                            this.storePartialMatchData(this.repositoryPath, chain, match.address, compilationResult, source.solidity);
                        }
                        else {
                            err = new Error("Could not match on-chain deployed bytecode to recompiled bytecode for:\n" +
                                (JSON.stringify(target, null, ' ') + "\n") +
                                "Addresses checked:\n" +
                                ("" + JSON.stringify(addresses, null, ' ')));
                            this.log.info({
                                loc: '[INJECT]',
                                chain: chain,
                                addresses: addresses,
                                err: err
                            });
                            throw new Error(err.message);
                        }
                        _a.label = 9;
                    case 9:
                        _i++;
                        return [3 /*break*/, 1];
                    case 10: return [2 /*return*/, match];
                }
            });
        });
    };
    /**
   * Writes verified sources to repository by address and by ipfs | swarm hash
   * @param {string}              repository        repository root (ex: 'repository')
   * @param {string}              chain             chain name (ex: 'ropsten')
   * @param {string}              address           contract address
   * @param {RecompilationResult} compilationResult solc output
   * @param {StringMap}           sources           'rearranged' sources
   */
    Injector.prototype.storePerfectMatchData = function (repository, chain, address, compilationResult, sources) {
        var metadataPath;
        var bytes = web3_1.default.utils.hexToBytes(compilationResult.deployedBytecode);
        var cborData = core_1.cborDecode(bytes);
        if (cborData['bzzr0']) {
            metadataPath = "/swarm/bzzr0/" + web3_1.default.utils.bytesToHex(cborData['bzzr0']).slice(2);
        }
        else if (cborData['bzzr1']) {
            metadataPath = "/swarm/bzzr1/" + web3_1.default.utils.bytesToHex(cborData['bzzr1']).slice(2);
        }
        else if (cborData['ipfs']) {
            metadataPath = "/ipfs/" + multihashes.toB58String(cborData['ipfs']);
        }
        else {
            var err = new Error("Re-compilation successful, but could not find reference to metadata file in cbor data.");
            this.log.info({
                loc: '[STOREDATA]',
                address: address,
                chain: chain,
                err: err
            });
            throw err;
        }
        var hashPath = path_1.default.join(repository, metadataPath);
        var addressPath = path_1.default.join(repository, 'contracts', 'full_match', chain, address, '/metadata.json');
        this.fileService.save(hashPath, compilationResult.metadata);
        this.fileService.save(addressPath, compilationResult.metadata);
        for (var sourcePath in sources) {
            var sanitizedPath = sourcePath
                .replace(/[^a-z0-9_.\/-]/gim, "_")
                .replace(/(^|\/)[.]+($|\/)/, '_');
            var outputPath = path_1.default.join(repository, 'contracts', 'full_match', chain, address, 'sources', sanitizedPath);
            this.fileService.save(outputPath, sources[sourcePath]);
        }
    };
    /**
     * Writes verified sources to repository by address under the "partial_match" folder.
     * This method used when recompilation bytecode matches deployed *except* for their
     * metadata components.
     * @param {string}              repository        repository root (ex: 'repository')
     * @param {string}              chain             chain name (ex: 'ropsten')
     * @param {string}              address           contract address
     * @param {RecompilationResult} compilationResult solc output
     * @param {StringMap}           sources           'rearranged' sources
     */
    Injector.prototype.storePartialMatchData = function (repository, chain, address, compilationResult, sources) {
        var addressPath = path_1.default.join(repository, 'contracts', 'partial_match', chain, address, '/metadata.json');
        this.fileService.save(addressPath, compilationResult.metadata);
        for (var sourcePath in sources) {
            var sanitizedPath = sourcePath
                .replace(/[^a-z0-9_.\/-]/gim, "_")
                .replace(/(^|\/)[.]+($|\/)/, '_');
            var outputPath = path_1.default.join(repository, 'contracts', 'partial_match', chain, address, 'sources', sanitizedPath);
            this.fileService.save(outputPath, sources[sourcePath]);
        }
    };
    return Injector;
}());
exports.Injector = Injector;
//# sourceMappingURL=Injector.js.map