import Web3 from 'web3';
import { StringMap } from '@ethereum-sourcify/core';
export interface RecompilationResult {
    bytecode: string;
    deployedBytecode: string;
    metadata: string;
}
export declare interface ReformattedMetadata {
    input: any;
    fileName: string;
    contractName: string;
}
/**
 * Checks if provided endpoint is online
 * @param {string} provider personal project ID from infura.io or real Ethereum node endpoint
 */
export declare function checkEndpoint(provider: string): Promise<void>;
/**
 * Wraps eth_getCode
 * @param {Web3}   web3    connected web3 instance
 * @param {string} address contract
 */
export declare function getBytecode(web3: Web3, address: string): Promise<string>;
/**
 * Compiles sources using version and settings specified in metadata
 * @param  {any}                          metadata
 * @param  {string[]}                     sources  solidity files
 * @return {Promise<RecompilationResult>}
 */
export declare function recompile(metadata: any, sources: StringMap, log: any): Promise<RecompilationResult>;
/**
 * Removes post-fixed metadata from a bytecode string
 * (for partial bytecode match comparisons )
 * @param  {string} bytecode
 * @return {string}          bytecode minus metadata
 */
export declare function getBytecodeWithoutMetadata(bytecode: string): string;
