"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBytecodeWithoutMetadata = exports.recompile = exports.getBytecode = exports.checkEndpoint = void 0;
var web3_1 = __importDefault(require("web3"));
var node_fetch_1 = __importDefault(require("node-fetch"));
var solc = require('solc');
/**
 * Checks if provided endpoint is online
 * @param {string} provider personal project ID from infura.io or real Ethereum node endpoint
 */
function checkEndpoint(provider) {
    return __awaiter(this, void 0, void 0, function () {
        var web3;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!provider.includes("http")) return [3 /*break*/, 2];
                    web3 = new web3_1.default(provider);
                    return [4 /*yield*/, web3.eth.getNodeInfo().catch(function () {
                            throw new Error("Check your node");
                        })];
                case 1:
                    _a.sent();
                    return [3 /*break*/, 5];
                case 2:
                    if (!provider) return [3 /*break*/, 4];
                    return [4 /*yield*/, node_fetch_1.default("https://mainnet.infura.io/v3/" + provider, {
                            method: "post",
                            headers: {
                                'Accept': 'application/json',
                                'Content-Type': 'application/json'
                            },
                            body: JSON.stringify({ "jsonrpc": "2.0", "id": 1, "method": "eth_blockNumber", "params": [] })
                        })
                            .then(function (response) {
                            if (response.status == 401) {
                                throw new Error("Check your Infura ID");
                            }
                        }).catch(function () {
                            throw new Error("Check your Infura ID");
                        })];
                case 3:
                    _a.sent();
                    return [3 /*break*/, 5];
                case 4: throw new Error("No provider set");
                case 5: return [2 /*return*/];
            }
        });
    });
}
exports.checkEndpoint = checkEndpoint;
/**
 * Wraps eth_getCode
 * @param {Web3}   web3    connected web3 instance
 * @param {string} address contract
 */
function getBytecode(web3, address) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    address = web3.utils.toChecksumAddress(address);
                    return [4 /*yield*/, web3.eth.getCode(address)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.getBytecode = getBytecode;
;
/**
 * Compiles sources using version and settings specified in metadata
 * @param  {any}                          metadata
 * @param  {string[]}                     sources  solidity files
 * @return {Promise<RecompilationResult>}
 */
function recompile(metadata, sources, log) {
    return __awaiter(this, void 0, void 0, function () {
        var _a, input, fileName, contractName, version, solcjs, compiled, output, contract;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = reformatMetadata(metadata, sources, log), input = _a.input, fileName = _a.fileName, contractName = _a.contractName;
                    version = metadata.compiler.version;
                    log.info({
                        loc: '[RECOMPILE]',
                        fileName: fileName,
                        contractName: contractName,
                        version: version
                    }, 'Recompiling');
                    return [4 /*yield*/, new Promise(function (resolve, reject) {
                            solc.loadRemoteVersion("v" + version, function (error, soljson) {
                                (error) ? reject(error) : resolve(soljson);
                            });
                        })];
                case 1:
                    solcjs = _b.sent();
                    compiled = solcjs.compile(JSON.stringify(input));
                    output = JSON.parse(compiled);
                    contract = output.contracts[fileName][contractName];
                    return [2 /*return*/, {
                            bytecode: contract.evm.bytecode.object,
                            deployedBytecode: "0x" + contract.evm.deployedBytecode.object,
                            metadata: contract.metadata.trim()
                        }];
            }
        });
    });
}
exports.recompile = recompile;
/**
 * Removes post-fixed metadata from a bytecode string
 * (for partial bytecode match comparisons )
 * @param  {string} bytecode
 * @return {string}          bytecode minus metadata
 */
function getBytecodeWithoutMetadata(bytecode) {
    // Last 4 chars of bytecode specify byte size of metadata component,
    var metadataSize = parseInt(bytecode.slice(-4), 16) * 2 + 4;
    return bytecode.slice(0, bytecode.length - metadataSize);
}
exports.getBytecodeWithoutMetadata = getBytecodeWithoutMetadata;
/**
 * Formats metadata into an object which can be passed to solc for recompilation
 * @param  {any}                 metadata solc metadata object
 * @param  {string[]}            sources  solidity sources
 * @return {ReformattedMetadata}
 */
function reformatMetadata(metadata, sources, log) {
    var input = {};
    var fileName = '';
    var contractName = '';
    input.settings = metadata.settings;
    for (fileName in metadata.settings.compilationTarget) {
        contractName = metadata.settings.compilationTarget[fileName];
    }
    delete input['settings']['compilationTarget'];
    if (contractName == '') {
        var err = new Error("Could not determine compilation target from metadata.");
        log.info({ loc: '[REFORMAT]', err: err });
        throw err;
    }
    input['sources'] = {};
    for (var source in sources) {
        input.sources[source] = { 'content': sources[source] };
    }
    input.language = metadata.language;
    input.settings.metadata = input.settings.metadata || {};
    input.settings.outputSelection = input.settings.outputSelection || {};
    input.settings.outputSelection[fileName] = input.settings.outputSelection[fileName] || {};
    input.settings.outputSelection[fileName][contractName] = [
        'evm.bytecode',
        'evm.deployedBytecode',
        'metadata'
    ];
    return {
        input: input,
        fileName: fileName,
        contractName: contractName
    };
}
//# sourceMappingURL=utils.js.map